function [uc,ut]=SplitBregCarTex(f,mu1,mu2,lambda,tol,frame,Level,wLevel,maxit)

%[uc,ut]=SplitBregCarTex(f,mu1,mu2,lambda,tol,frame,Level,wLevel,maxit,oimg)
%Split Bregman iteration for Cartoon and texture separation. 
%The tight frame systems are framlets.
%Input:   f -- image to be separated;
%  mu1, mu2 -- regularization parameters for cartoon and texure respectively.
%    lambda -- parameter for split Bregman.
%     frame -- framelet system used. 0-Haar, 1-Piecewise Linear, 3-Picewise Cubic
%     Level -- framelet decomposition level.
%    wLevel -- weighting of thresholding on each level. If wLevel<=0, we 
%              use default values, the product of norms of filters. If 
%              wLevel>0, the thresholding is multiplied by wLevel from the 
%              current level to the next level. 

[D,R]=GenerateFrameletFilter(frame);
nD=length(D);
[m,n]=size(f);

LWSize=16; % Window size of the local DCT.
LFSize=-1; % local frequency in the local DCT.

%Compute the weighted thresholding parameters.
muLevel=getwThresh(mu1/lambda,wLevel,Level,D);

normf=norm(f,'fro');

%initialization
uc=zeros(m,n);
ut=zeros(m,n);
bc=FraDecMultiLevel(uc,D,Level);
bt=LDCT(ut,LWSize,LFSize);
dc=CoeffOper('=',bc);
dt=bt;

ddd=1/lambda/(2+lambda);

nD=length(D);

for nstep=1:maxit 

    %for innerstep=1:1
        % solve u.
        puc=f+lambda*FraRecMultiLevel(CoeffOper('-',dc,bc),R,Level);
        put=f+lambda*ILDCT2(dt-bt,LWSize);
        
        uc=(1+lambda)*ddd*puc-1*ddd*put;
        ut=(1+lambda)*ddd*put-1*ddd*puc;
        
        % solve d
        tcc=FraDecMultiLevel(uc,D,Level);
        dc=CoeffOper('+',tcc,bc);
        dc=CoeffOper('s',dc,muLevel);
        
        ttc=LDCT(ut,LWSize,LFSize);
        dt=ttc+bt;
        dt=wthresh(dt,'s',mu2/lambda);


        % update b
        errc=CoeffOper('-',tcc,dc);
        error=norm(FraRecMultiLevel(errc,R,Level),'fro')^2;
        bc=CoeffOper('+',bc,errc);
        
        errt=ttc-dt;
        error=error+norm(errt,'fro')^2;
        bt=bt+errt;
        
        error=sqrt(error)/normf;
        disp(['error on step ' num2str(nstep) ' is ' num2str(error)]);
        if error<tol
            break;
        end
    %end
end

function [D,R]=GenerateFrameletFilter(frame)
if frame==0          %Haar Wavelet
    D{1}=[0 1 1]/2;
    D{2}=[0 1 -1]/2;
    D{3}='cc';
    R{1}=[1 1 0]/2;
    R{2}=[-1 1 0]/2;
    R{3}='cc';
elseif frame==1      %Piecewise Linear Framelet
    D{1}=[1 2 1]/4;
    D{2}=[1 0 -1]/4*sqrt(2);
    D{3}=[-1 2 -1]/4;
    D{4}='ccc';
    R{1}=[1 2 1]/4;
    R{2}=[-1 0 1]/4*sqrt(2);
    R{3}=[-1 2 -1]/4;
    R{4}='ccc';
elseif frame==3      %Piecewise Cubic Framelet
    D{1}=[1 4 6 4 1]/16;
    D{2}=[1 2 0 -2 -1]/8;
    D{3}=[-1 0 2 0 -1]/16*sqrt(6);
    D{4}=[-1 2 0 -2 1]/8;
    D{5}=[1 -4 6 -4 1]/16;
    D{6}='ccccc';
    R{1}=[1 4 6 4 1]/16;
    R{2}=[-1 -2 0 2 1]/8;
    R{3}=[-1 0 2 0 -1]/16*sqrt(6);
    R{4}=[1 -2 0 2 -1]/8;
    R{5}=[1 -4 6 -4 1]/16;
    R{6}='ccccc';
end

function muLevel=getwThresh(mu,wLevel,Level,D);
nfilter=1;
nD=length(D);
if wLevel<=0
    for ki=1:Level
        for ji=1:nD-1
            for jj=1:nD-1
                muLevel{ki}{ji,jj}=mu*nfilter*norm(D{ji})*norm(D{jj});
            end
        end
        nfilter=nfilter*norm(D{1});
    end
else
    for ki=1:Level
        for ji=1:nD-1
            for jj=1:nD-1
                muLevel{ki}{ji,jj}=mu*nfilter*norm(D{ji})*norm(D{jj});
            end
        end
        nfilter=nfilter*wLevel;
    end
end
muLevel{Level}{1,1}=0;

function gamma=CoeffOper(op,alpha,beta)
% This subroutine implement alpha op beta = gamma, where op= + - = * s n
Level=length(alpha);
[nD,nD1]=size(alpha{1});
for ki=1:Level
    for ji=1:nD
        for jj=1:nD
            if op=='='
                gamma{ki}{ji,jj}=alpha{ki}{ji,jj};
            elseif op=='-'
                gamma{ki}{ji,jj}=alpha{ki}{ji,jj}-beta{ki}{ji,jj};
            elseif op=='+'
                gamma{ki}{ji,jj}=alpha{ki}{ji,jj}+beta{ki}{ji,jj};
            elseif op=='*'
                gamma{ki}{ji,jj}=alpha{ki}{ji,jj}*beta;
            elseif op=='s'
                gamma{ki}{ji,jj}=wthresh(alpha{ki}{ji,jj},'s',beta{ki}{ji,jj});
            end
        end
    end
end